#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from rich import print
ROOT_PATH = os.path.join(sys.path[0], '../')
sys.path.append(ROOT_PATH)

import click
import shutil
import importlib
import configparser
from msds.interact import interact
from msds.core.getBackTest import go_back_test
from msds.core.getCandleData import save_spot_candle_data_from_exchange

config = configparser.ConfigParser()
config.read([os.path.join(os.getcwd(), 'msds.ini')])

# ------------------------------ 命令定义 ------------------------------
@click.group()
def msds():
    pass

@msds.command()
def init():
    origin_file = ''
    # print(origin_file)
    print(sys.path)
    target_file = os.path.join(os.getcwd(), 'msds.ini')
    if not os.path.exists(target_file):
        shutil.copyfile(origin_file, target_file)
    else:
        print('[bold red]已存在配置文件[/bold red]')

@msds.command()
def data():
    if not bool(config.has_section('CANDLE_DATA')):
        exchange, symbol, time_start, time_end, time_interval = interact.userAnswerData()
        save_spot_candle_data_from_exchange(exchange, symbol, time_start, time_end, time_interval)
    candleConfig = config['CANDLE_DATA']
    return save_spot_candle_data_from_exchange(
        candleConfig['exchange'],
        candleConfig['symbol'],
        candleConfig['time_start'],
        candleConfig['time_end'],
        candleConfig['time_interval']
    )

@msds.command()
def backtest():
    if not bool(config.has_section('BACK_TEST')):
        return
    BackTestConfig = config['BACK_TEST']

    # 配置引入
    strategy_name = BackTestConfig['strategy_name'] or 'main'
    strategy_path = BackTestConfig['strategy_path'] or os.getcwd()
    sys.path.append(strategy_path)
    strategy = importlib.import_module(strategy_name)
    go_back_test(
        strategy,
        eval(BackTestConfig['strategy_param']),
        BackTestConfig['strategy_time'],
        BackTestConfig['data_path'],
        BackTestConfig.getint('symbol_face_value'),
        BackTestConfig.getfloat('service_charge'),
        BackTestConfig.getfloat('slippage'),
        BackTestConfig.getint('leverage_rate'),
        BackTestConfig.getfloat('min_margin_ratio'),
    )

@msds.command()
def test():
    pass

# ------------------------------ 服务入口 ------------------------------

def main():
    msds()

# 描述：只有脚本执行的方式时运行main函数
# case1、当该模块被直接执行时：__name__ 等于文件名（包含后缀 .py ）
# case2、当该模块 import 到其他模块中： __name__ 等于模块名称（不包含后缀.py）
if __name__ == '__main__':
    main()



import pandas as pd
import random


# 将None作为信号返回
def real_signal_none(df, para):
    """
    发出空交易信号
    :param df:
    :param para:
    :return:
    """

    return None


# 随机生成交易信号
def real_signal_random(df,  para=[200,2]):
    """
    随机发出交易信号
    :param df:
    :param para:
    :return:
    """

    r = random.random()
    if r <= 0.25:
        return 1
    elif r <= 0.5:
        return -1
    elif r <= 0.75:
        return 0
    else:
        return None


# 布林策略实盘交易信号
def signal_simple_bolling(df, para=[550, 2.1, 0.05]):
    """
    :param df:
    :param para: n, m
    :return:

    # 布林线策略
    # 布林线中轨：n天收盘价的移动平均线
    # 布林线上轨：n天收盘价的移动平均线 + m * n天收盘价的标准差
    # 布林线下轨：n天收盘价的移动平均线 - m * n天收盘价的标准差
    # 当收盘价由下向上穿过上轨的时候，做多；然后由上向下穿过中轨的时候，平仓。
    # 当收盘价由上向下穿过下轨的时候，做空；然后由下向上穿过中轨的时候，平仓。
    """

    # ===策略参数
    n = int(para[0])
    m = para[1]
    p = para[2]

    # ===计算指标
    # 计算均线
    df['median'] = df['close'].rolling(n, min_periods=1).mean()
    # 计算上轨、下轨道
    df['std'] = df['close'].rolling(n, min_periods=1).std(ddof=0)  # ddof代表标准差自由度
    df['upper'] = df['median'] + m * df['std']
    df['lower'] = df['median'] - m * df['std']
    df['abs'] = abs(df['close'] / df['median'] - 1)

    # ===计算信号
    # 找出做多信号
    condition1 = df['close'] > df['upper']  # 当前K线的收盘价 > 上轨
    condition2 = df['close'].shift(1) <= df['upper'].shift(1)  # 之前K线的收盘价 <= 上轨
    condition3 = df['abs'] <= p
    df.loc[condition1 & condition3, 'signal_long'] = 1  # 将产生做多信号的那根K线的signal设置为1，1代表做多

    # 找出做多平仓信号
    condition1 = df['close'] < df['median']  # 当前K线的收盘价 < 中轨
    condition2 = df['close'].shift(1) >= df['median'].shift(1)  # 之前K线的收盘价 >= 中轨
    df.loc[condition1 & condition2, 'signal_long'] = 0  # 将产生平仓信号当天的signal设置为0，0代表平仓

    # 找出做空信号
    condition1 = df['close'] < df['lower']  # 当前K线的收盘价 < 下轨
    condition2 = df['close'].shift(1) >= df['lower'].shift(1)  # 之前K线的收盘价 >= 下轨
    condition3 = df['abs'] <= p
    df.loc[condition1 & condition3, 'signal_short'] = -1  # 将产生做空信号的那根K线的signal设置为-1，-1代表做空

    # 找出做空平仓信号
    condition1 = df['close'] > df['median']  # 当前K线的收盘价 > 中轨
    condition2 = df['close'].shift(1) <= df['median'].shift(1)  # 之前K线的收盘价 <= 中轨
    df.loc[condition1 & condition2, 'signal_short'] = 0  # 将产生平仓信号当天的signal设置为0，0代表平仓

    # 合并做多做空信号，去除重复信号
    df['signal'] = df[['signal_long', 'signal_short']].sum(axis=1, min_count=1, skipna=True)  # 若你的pandas版本是最新的，请使用本行代码代替上面一行
    temp = df[df['signal'].notnull()][['signal']]
    temp = temp[temp['signal'] != temp['signal'].shift(1)]
    df['signal'] = temp['signal']

    # ===删除无关变量
    df.drop(['median', 'std', 'upper', 'lower', 'signal_long', 'signal_short'], axis=1, inplace=True)

    return df


import pandas as pd
from msds.func.Signals import *
from msds.func.Position import *
from msds.func.Evaluate import *
from datetime import timedelta
pd.set_option('expand_frame_repr', False)  # 当列太多时不换行

def go_back_test(strategy, params, rule_type, data_path, face_value, c_rate, slippage, leverage_rate, min_margin_ratio):

    # =====读入数据
    df = pd.read_hdf(data_path, key='df')
    # 任何原始数据读入都进行一下排序、去重，以防万一
    df.sort_values(by=['candle_begin_time'], inplace=True)
    df.drop_duplicates(subset=['candle_begin_time'], inplace=True)
    df.reset_index(inplace=True, drop=True)


    # =====转换为其他分钟数据
    period_df = df.resample(rule=rule_type, on='candle_begin_time', label='left', closed='left').agg(
        {'open': 'first',
        'high': 'max',
        'low': 'min',
        'close': 'last',
        'volume': 'sum'
    })
    period_df.dropna(subset=['open'], inplace=True)  # 去除一天都没有交易的周期
    period_df = period_df[period_df['volume'] > 0]  # 去除成交量为0的交易周期
    period_df.reset_index(inplace=True)
    df = period_df[['candle_begin_time', 'open', 'high', 'low', 'close', 'volume']]
    df = df[df['candle_begin_time'] >= pd.to_datetime('2021-6-01')]
    df = df[df['candle_begin_time'] < pd.to_datetime('2021-9-30')]
    df.reset_index(inplace=True, drop=True)


    # =====计算交易信号
    df = strategy.main(df, para=params)

    # =====计算实际持仓
    df = position_for_OKEx_future(df)

    # =====计算资金曲线
    # 选取相关时间。币种上线10天之后的日期
    # t = df.iloc[0]['candle_begin_time'] + timedelta(days=drop_days)
    # df = df[df['candle_begin_time'] > t]
    # 计算资金曲线
    df = equity_curve_for_OKEx_USDT_future_next_open(df, slippage=slippage, c_rate=c_rate, leverage_rate=leverage_rate,
                                                    face_value=face_value, min_margin_ratio=min_margin_ratio)
    print(df)
    print('策略最终收益：', df.iloc[-1]['equity_curve'])

    # return df.iloc[-1]['equity_curve']